# bbi_paths.lib
Version=3.5.1

#==============================================================================
# Copyright and license info is available in the LICENSE file included with
# the P4BBI tool, and also available online:
# https://swarm.workshop.perforce.com/projects/perforce-software-baseline-and-branch-import/view/main/LICENSE
#------------------------------------------------------------------------------

#==============================================================================
# BBI Library Functions.

#------------------------------------------------------------------------------
# Function: get_stream_subdir ($path)
# Given:                             Return:
#    //stream/main/...               "" (empty)
#    //stream/main/ACE/...           ACE
#    //stream/main/ACE/d1/d2/...     ACE/d1/d2

function get_stream_subdir () {
   # Write debug to stderr to avoid corrupting return value on stdout.
   [[ $VERBOSITY -ge 3 ]] && echo "CALL: get_stream_subdir ($*)" >&2

   declare path=${1:-""}
   declare streamDepth=${2:-1}
   declare -i i=0

   # Replace trailing '/...' with '/'.
   path=${path/\/.../\/}

   # Trim leading '//*/*'.
   path=${path#//*/*/}

   if [[ $streamDepth =~ ^[0-9]+$ && $streamDepth -ge 1 ]]; then
      # Trim one directory level per level of stream depth.
      i=1; while [[ $i -lt $streamDepth ]]; do
         path=${path#*/}
         i=$((i+1))
      done
   fi

   # Trim trailing '/'.
   path=${path%/}

   echo $path
}

#------------------------------------------------------------------------------
# Function: get_stream ($path)
#
# Given:                             Return:
#    //stream/main/...               //stream/main
#    //stream/main/ACE/...           //stream/main
#    //stream/main/ACE/d1/d2/...     //stream/main
#    //stream/main                   //stream/main

function get_stream () {
   # Write debug to stderr to avoid corrupting return value on stdout.
   [[ $VERBOSITY -ge 3 ]] && echo "CALL: get_stream ($*)" >&2

   declare path=${1:-""}
   declare streamDepth=${2:-1}
   declare -i i=0 fieldNum=0
   declare stream= nextDir

   # Trim trailing '/...'.
   path=${path/\/...//}

   # Replace '/' with ':' (for easier field separation).
   path=$(echo $path | sed "s:/:\::g")

   stream="//$(echo $path|cut -d ':' -f 3)/$(echo $path|cut -d ':' -f 4)"

   if [[ $streamDepth =~ ^[0-9]+$ && $streamDepth -ge 1 ]]; then
      i=1; while [[ $i -lt $streamDepth ]]; do
         fieldNum=$((i+4))
         nextDir="$(echo $path|cut -d ':' -f $fieldNum)"
         if [[ -n "$nextDir" ]]; then
            stream="$stream/$nextDir"
         else
            break
         fi
         i=$((i+1))
      done
   fi

   echo $stream
}
